@@ -8,7 +8,9 @@
 import collections, inspect
 from sqlalchemy.orm import scoped_session, sessionmaker, relationship, backref
 from sqlalchemy.ext.declarative import declarative_base, declared_attr
-from sqlalchemy import create_engine, Column, Integer, String, DateTime, ForeignKey, func
+from sqlalchemy import create_engine, Column, Integer, DateTime, ForeignKey, func
+from sqlalchemy.schema import Table
+from types import MethodType
 
 
 class Database:
@@ -46,6 +48,10 @@ Please specify something like '?charset=utf8' explicitly.""")
 
         # If you want to deal with orm, you need session 
         self.session = scoped_session(sessionmaker(autocommit = False, autoflush = False, bind = self.engine))
+        def add_then_commit(session, obj):
+            session.add(obj)
+            session.commit()        
+        self.session.add_then_commit = MethodType(add_then_commit, self.session)
 
     def create_tables(self):
         """Create all tables, make sure:
@@ -78,27 +84,52 @@ Please specify something like '?charset=utf8' explicitly.""")
             raise TypeError('parameter data is of the wrong type')
 
     @staticmethod
-    def foreign_key(table, foreign_name = None, backref_name = None):
+    def foreign_key(foreign_table_name, foreign_name = None, backref_name = None):
         """"Class decorator, add a foreign key to a SQLAlchemy model.
-        Parameter table is the destination table, in a one-to-many relationship, table is the "one" side.
+        Parameter table_name is the destination table, in a one-to-many relationship, it is the "one" side.
         foreign_name is the user-friendly name of table(if omitted, table will be used instead)
         backref_name is the name used to back ref the "many" side.
         """
-        foreign_name = foreign_name or table
+        foreign_model_name = foreign_table_name.capitalize()
+        foreign_name = foreign_name or foreign_table_name
         foreign_key = '{0}_id'.format(foreign_name)        
         def ref_table(cls):
-            setattr(cls, foreign_key, Column(Integer, ForeignKey('{0}.id'.format(table))))
+            model_name = cls.__name__
+            table_name = model_name.lower()
+            setattr(cls, foreign_key, Column(Integer, ForeignKey('{0}.id'.format(foreign_table_name))))
             # Why create a new variable "my_backref_name" here?
             # Because, all operations that introduce new names use the local scope, especially assignments.
             # Assign to backref_name will change it to a local variable which we don't want to, so we have to create a new variable.
             # reference: http://docs.python.org/tutorial/classes.html#python-scopes-and-namespaces
-            my_backref_name = backref_name or '{0}s'.format(cls.__name__.lower())
-            setattr(cls, foreign_name, relationship(table.capitalize(), 
-                primaryjoin = '{0}.{1} == {2}.id'.format(cls.__name__, foreign_key, table.capitalize()), 
+            my_backref_name = backref_name or '{0}s'.format(table_name)
+            setattr(cls, foreign_name, relationship(foreign_model_name, 
+                primaryjoin = '{0}.{1} == {2}.id'.format(model_name, foreign_key, foreign_model_name), 
                 backref = backref(my_backref_name, lazy = 'dynamic')))
             return cls
         return ref_table
 
+    @staticmethod
+    def many_to_many(ref_table_name, ref_name = None, backref_name = None):
+        """Class Decorator, add a many-to-many relationship between two SQLAlchemy models.
+        Parameter table is the destination table, it is NOT the one which this method decorated on.
+        ref_name is how this model reference the destination models
+        backref_name is how the destination model reference this model.
+        """
+        ref_name = ref_name or '{0}s'.format(ref_table_name)
+        ref_model_name = ref_table_name.capitalize()
+        def ref_table(cls):
+            table_name = cls.__name__.lower()            
+            middle_table = Table(
+                '{0}_{1}'.format(table_name, ref_table_name), Database.Base.metadata,
+                Column('{0}_id'.format(table_name), Integer, ForeignKey('{0}.id'.format(table_name))),
+                Column('{0}_id'.format(ref_table_name), Integer, ForeignKey('{0}.id'.format(ref_table_name))),
+            )
+            my_backref_name = backref_name or '{0}s'.format(table_name)
+            setattr(cls, ref_name, relationship(ref_model_name, secondary = middle_table, 
+                lazy = 'dynamic', backref = backref(my_backref_name, lazy = 'dynamic')))
+            return cls
+        return ref_table
+
     @staticmethod
     def created_timestamp(cls):
         """Class decorator, add a timestamp field 'created' to the model"""
